package com.fr.plugin.widget.digitroll;

import com.fr.base.Formula;
import com.fr.base.ParameterMapNameSpace;
import com.fr.form.data.DataBinding;
import com.fr.form.ui.DataControl;
import com.fr.form.ui.DirectWriteEditor;
import com.fr.form.ui.Widget;
import com.fr.form.ui.WidgetValue;
import com.fr.form.ui.concept.data.ValueInitializer;
import com.fr.json.JSONArray;
import com.fr.json.JSONException;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.script.Calculator;
import com.fr.stable.ArrayUtils;
import com.fr.stable.CodeUtils;
import com.fr.stable.core.NodeVisitor;
import com.fr.stable.script.CalculatorProvider;
import com.fr.stable.web.Repository;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLableReader;
import com.fr.web.core.TemplateSessionIDInfo;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import java.awt.*;
import java.math.BigInteger;


public class DigitRollWidget extends DirectWriteEditor implements DataControl {

    public static final String XTYPE = "digitroll";
    public static final String DURATION = "duration";
    public static final String FONT_SIZE = "fontSize";
    public static final String FONT_COLOR = "fontColor";
    public static final String BACKGROUND_COLOR = "backgroundColor";
    public static final String PREFIX = "prefix";
    public static final String SUFFIX = "suffix";
    public static final String GROUP = "group";
    public static final String DECIMAL_PALACES = "decimalPlaces";
    public static final String DECIMAL = "decimal";

    private ValueInitializer widgetValue;
    private int duration;
    private int fontSize;
    private Object fontColor;
    private Object backgroundColor;
    private String prefix;
    private String suffix;
    private boolean group;
    private int decimalPlaces;

    public DigitRollWidget() {
        widgetValue = new WidgetValue();
        duration = 0;
        fontSize = 70;
        fontColor = new Color(204, 204, 204);
        backgroundColor = new Color(51, 51, 51);
        prefix = "";
        suffix = "";
        setGroup(true);
        setDecimalPlaces(0);
    }

    public boolean isGroup() {
        return group;
    }

    public void setGroup(boolean group) {
        this.group = group;
    }

    public int getDecimalPlaces() {
        return decimalPlaces;
    }

    public void setDecimalPlaces(int decimalPlaces) {
        this.decimalPlaces = decimalPlaces;
    }

    public int getDuration() {
        return this.duration;
    }

    public void setDuration(int duration) {
        this.duration = duration;
    }

    public ValueInitializer getWidgetValue() {
        return this.widgetValue;
    }

    public void setWidgetValue(ValueInitializer widgetValue) {
        this.widgetValue = widgetValue;
    }

    public int getFontSize() {
        return fontSize;
    }

    public void setFontSize(int fontSize) {
        this.fontSize = fontSize;
    }

    public Object getFontColor() {
        return fontColor;
    }

    public void setFontColor(Object fontColor) {
        this.fontColor = fontColor;
    }

    public Object getBackgroundColor() {
        return backgroundColor;
    }

    public void setBackgroundColor(Object backgroundColor) {
        this.backgroundColor = backgroundColor;
    }

    public String getPrefix() {
        return prefix;
    }

    public void setPrefix(String prefix) {
        this.prefix = prefix;
    }

    public String getSuffix() {
        return suffix;
    }

    public void setSuffix(String suffix) {
        this.suffix = suffix;
    }

    @Override
    public String getXType() {//控件类型，参考其他的案例，返回的是文本框
        return XTYPE;
    }

    @Override
    public int[] getValueType() {
        //return new int[]{TYPE_FORMULA, TYPE_DATABINDING};
        return new int[]{TYPE_DATABINDING};
    }

    @Override
    public String[] dependence(CalculatorProvider calculatorProvider) {
        return super.dependence(calculatorProvider);
    }

    @Override
    public String getDataBindDefaultValue(CalculatorProvider calculatorProvider) {
        return null;
    }

    /**
     * 读取插件的值
     */
    private int dataSetCursor = 0;

    private double getWidgetValue(Calculator calculator) throws Exception {
        double value = 0;
        if (this.widgetValue.getValue() == null) {
            value = 0;
        } else if (this.widgetValue.getValue() instanceof Formula) {
            Object v = calculator.eval((Formula) this.widgetValue.getValue());
            value = Util.valueChangeToDouble(v);
        } else if (this.widgetValue.getValue() instanceof DataBinding) {
            calculator.setAttribute(Widget.NAME, this.getWidgetName().toUpperCase());
            calculator.setAttribute(ValueInitializer.RETURN_ARRAY, true);
            calculator.setAttribute(ValueInitializer.VALUE_REPEAT, true);
            Object r = getWidgetValue().executeResult(calculator);
            if (r instanceof Object[]) {
                Object[] ra = (Object[]) r;
                dataSetCursor %= ra.length;
                Object val = ra[dataSetCursor++];
                return Util.valueChangeToDouble(val);
            } else {
                return (Util.valueChangeToDouble(r));
            }
        }
        return value;
    }

    @Override
    public void createValueResult(CalculatorProvider calculatorProvider, JSONObject jsonObject) {
        Object result = null;
        try {
            if (widgetValue.getValue() instanceof Formula) {
                result = calculatorProvider.eval((Formula) widgetValue.getValue());
            } else if (widgetValue.getValue() instanceof DataBinding) {
                calculatorProvider.setAttribute(Widget.NAME, this.getWidgetName().toUpperCase());
                calculatorProvider.setAttribute(ValueInitializer.RETURN_ARRAY, true);
                calculatorProvider.setAttribute(ValueInitializer.VALUE_REPEAT, true);
                result = getWidgetValue().executeResult(calculatorProvider);
            }
            jsonObject.put(this.widgetName.toUpperCase(), result == null ? 0 : result);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
    }

    @Override
    public String getFormatText() {
        return "";
    }

    @Override
    public JSONObject createJSONConfig(Repository repository,
                                       Calculator calculator,
                                       NodeVisitor visitor) throws JSONException {
        JSONObject jsonConfig = super.createJSONConfig(repository, calculator, visitor);
        Object value = jsonConfig.get("value");
        if (value != null) {
            Double v = 0d;
            try {
                JSONArray jsonArray = (JSONArray) value;
                v = jsonArray.getDouble(0);
            } catch (Exception e) {
                if (value != null) {
                    if (value instanceof BigInteger || value instanceof Integer) {
                        String s = value.toString();
                        v = Double.valueOf(s);
                    } else {
                        v = (Double) value;
                    }
                } else {
                    FineLoggerFactory.getLogger().error(e.getMessage(), e);
                }
            }
            jsonConfig.put("value", v);
        }

        jsonConfig.put(DURATION, this.value2Config(this.duration, calculator));
        jsonConfig.put(PREFIX, getPrefix());
        jsonConfig.put(SUFFIX, getSuffix());
        jsonConfig.put(FONT_COLOR, Util.colorValueToHexString(((Color) getFontColor()).getRGB()));
        jsonConfig.put(BACKGROUND_COLOR, Util.colorValueToHexString(((Color) getBackgroundColor()).getRGB()));
        jsonConfig.put(FONT_SIZE, getFontSize());
        jsonConfig.put(GROUP, isGroup());
        //jsonConfig.put(DECIMAL, (getDecimalPlaces() <= 0 ? false : true));
        jsonConfig.put(DECIMAL_PALACES, getDecimalPlaces());

        String[] dependence = this.dependence(calculator);
        if (!ArrayUtils.isEmpty(dependence)) {
            jsonConfig.put("dependence", dependence);
        }
        return jsonConfig;
    }

    @Override
    public JSONArray createJSONData(TemplateSessionIDInfo sessionIDInfor,
                                    Calculator calculator, HttpServletRequest request) throws Exception {
        JSONArray jsonArray;
        jsonArray = super.createJSONData(sessionIDInfor, calculator, request);
        ParameterMapNameSpace parameterMapNameSpace = ParameterMapNameSpace.create(sessionIDInfor.getParameterMap4Execute());
        DependenceNameSpace dependenceNameSpace = new DependenceNameSpace(CodeUtils.cjkDecode(WebUtils.getHTTPRequestParameter(request, "dependence")));
        calculator.pushNameSpace(parameterMapNameSpace);
        calculator.pushNameSpace(dependenceNameSpace);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("value", this.getWidgetValue(calculator));
        jsonArray.put(jsonObject);
        calculator.removeNameSpace(parameterMapNameSpace);
        calculator.removeNameSpace(dependenceNameSpace);
        return jsonArray;
    }

    @Override
    public String[] supportedEvents() {
        return new String[0];
    }

    @Override
    public void readXML(XMLableReader reader) {
        super.readXML(reader);
        if (reader.isChildNode()) {
            String tagName = reader.getTagName();
            if (tagName.equals("DigitRollAttr")) {
                String s = null;
                if ((s = reader.getAttrAsString(DURATION, null)) != null) {
                    this.setDuration(Integer.parseInt(s));
                }
                if ((s = reader.getAttrAsString(FONT_SIZE, null)) != null) {
                    setFontSize(Integer.parseInt(s));
                }

                int color;
                if ((color = reader.getAttrAsInt(FONT_COLOR, -1)) != -1) {
                    setFontColor(new Color(color));
                }
                if ((color = reader.getAttrAsInt(BACKGROUND_COLOR, -1)) != -1) {
                    setBackgroundColor(new Color(color));
                }
                setPrefix(reader.getAttrAsString(PREFIX, ""));
                setSuffix(reader.getAttrAsString(SUFFIX, ""));
                setDecimalPlaces(reader.getAttrAsInt(DECIMAL_PALACES, 0));
                setGroup(reader.getAttrAsBoolean(GROUP, true));

            } else if ("widgetValue".equals(tagName)) {
                this.widgetValue = new WidgetValue();
                reader.readXMLObject(this.widgetValue);
            }
        }
    }

    @Override
    public void writeXML(XMLPrintWriter writer) {
        super.writeXML(writer);
        writer.startTAG("DigitRollAttr");
        if (this.duration != 0) {
            writer.attr(DURATION, this.getDuration());
        }

        writer.attr(FONT_SIZE, getFontSize());
        writer.attr(FONT_COLOR, ((Color) getFontColor()).getRGB());
        writer.attr(BACKGROUND_COLOR, ((Color) getBackgroundColor()).getRGB());
        writer.attr(PREFIX, getPrefix());
        writer.attr(SUFFIX, getSuffix());
        writer.attr(GROUP, isGroup());
        writer.attr(DECIMAL_PALACES, getDecimalPlaces());
        writer.end();

        if (this.widgetValue != null) {
            this.widgetValue.writeXML(writer);
        }
    }
}
