package com.fr.plugin.widget.digitroll.ui;

import com.fr.design.Exception.ValidationException;
import com.fr.design.editor.ValueEditorPane;
import com.fr.design.editor.editor.Editor;
import com.fr.design.editor.editor.FormulaEditor;
import com.fr.design.mainframe.widget.editors.AbstractPropertyEditor;
import com.fr.design.mainframe.widget.editors.DataBindingEditor;
import com.fr.design.mainframe.widget.editors.ServerDataBindingEditor;
import com.fr.form.ui.DataControl;
import com.fr.form.ui.WidgetValue;
import com.fr.general.Inter;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

public class DigitRollDataSourceEditor extends AbstractPropertyEditor {
    private DataControl widget;
    private ValueEditorPane pane;

    public DigitRollDataSourceEditor(Object o) {
        this.widget = (DataControl) o;
        Editor[] editors = createWidgetValueEditor(this.widget, false);
        Editor[] editors1 = editors;
        int length = editors.length;

        for (int i = 0; i < length; ++i) {
            Editor e = editors1[i];
            e.addChangeListener(new ChangeListener() {
                public void stateChanged(ChangeEvent e) {
                    DigitRollDataSourceEditor.this.firePropertyChanged();
                }
            });
        }

        this.pane = new ValueEditorPane(editors);
        this.pane.addPropertyChangeListener("value", new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent evt) {
                DigitRollDataSourceEditor.this.firePropertyChanged();
            }
        });
    }

    public static Editor[] createWidgetValueEditor(DataControl data, boolean onlyServer) {
        int[] types = data.getValueType();
        Editor[] editor = new Editor[types.length];

        for (int i = 0; i < types.length; ++i) {
            editor[i] = createWidgetValueEditorByType(types[i], onlyServer);
        }

        return editor;
    }

    public static Editor createWidgetValueEditorByType(int type, boolean onlyServer) {
        switch (type) {
            case DataControl.TYPE_DATABINDING:
                return (Editor) (onlyServer ? new ServerDataBindingEditor() : new DataBindingEditor());
            case DataControl.TYPE_FORMULA:
                return new FormulaEditor(Inter.getLocText("Plugin-DigitRoll_Formula"));
            default:
                return null;
        }
    }

    public void validateValue() throws ValidationException {
    }

    public Component getCustomEditor() {
        return this.pane;
    }

    public Object getValue() {
        return new WidgetValue(this.pane.update());
    }

    public void setValue(Object value) {
        if (value != null) {
            this.pane.populate(((WidgetValue) value).getValue());
        }
    }
}
